require 'spec_helper_puppet'

describe 'jboss', :type => :class do
  let(:facts) { Testing::RspecPuppet::SharedFacts.oraclelinux_facts }
  extend Testing::RspecPuppet::SharedExamples
  describe 'with defaults for all parameters' do
    it { is_expected.to compile }
    it do
      is_expected.to contain_class('jboss').with(
        :product => 'wildfly',
        :version => '9.0.2.Final'
      )
    end
    it { is_expected.to contain_anchor 'jboss::begin' }
    it { is_expected.to contain_anchor 'jboss::end' }
    it { is_expected.to contain_anchor 'jboss::configuration::begin' }
    it { is_expected.to contain_anchor 'jboss::configuration::end' }
    it { is_expected.to contain_anchor 'jboss::installed' }
    it { is_expected.to contain_anchor 'jboss::package::begin' }
    it { is_expected.to contain_anchor 'jboss::package::end' }
    it { is_expected.to contain_anchor 'jboss::service::begin' }
    it { is_expected.to contain_anchor 'jboss::service::end' }
    it { is_expected.to contain_anchor 'jboss::service::started' }
    it { is_expected.to contain_user 'jboss' }
    it { is_expected.to contain_group 'jboss' }
    it do
      is_expected.to contain_class('jboss::internal::package').with(
        :version      => '9.0.2.Final',
        :product      => 'wildfly',
        :jboss_user   => 'jboss',
        :jboss_group  => 'jboss',
        :java_version => 'latest'
      )
    end
  end
  describe 'with product => jboss-eap and version => 6.4.0.GA parameters set' do
    let(:params) do
      { :product => 'jboss-eap', :version => '6.4.0.GA' }
    end

    it { is_expected.to compile }
    it { is_expected.to contain_class 'jboss' }
    it { is_expected.to contain_user 'jboss' }
    it { is_expected.to contain_group 'jboss' }
  end
  describe 'with jboss_user => appserver parameter set' do
    let(:params) do
      { :jboss_user => 'appserver' }
    end

    it { is_expected.to compile }
    it { is_expected.to contain_class 'jboss' }
    it { is_expected.to contain_user 'appserver' }
    it { is_expected.to contain_group 'jboss' }
  end
  describe 'with download_url => file:///tmp/wildfly-8.2.0.Final.zip set' do
    let(:params) do
      { :download_url => 'file:///tmp/wildfly-8.2.0.Final.zip' }
    end

    it do
      is_expected.to contain_class('jboss').with(
        :download_url => 'file:///tmp/wildfly-8.2.0.Final.zip'
      )
    end
    it { is_expected.to contain_class 'jboss::params' }
    it { is_expected.to contain_class 'jboss::internal::compatibility' }
    it { is_expected.to contain_class 'jboss::internal::configuration' }
    it { is_expected.to contain_class 'jboss::internal::service' }

    it_behaves_like common_interfaces
    it_behaves_like common_anchors
  end
end
